package techreborn.tiles;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import org.apache.commons.lang3.tuple.Pair;
import reborncore.api.tile.IInventoryProvider;
import reborncore.common.powerSystem.TilePowerAcceptor;
import reborncore.common.util.Inventory;
import reborncore.common.util.ItemUtils;
import techreborn.client.container.IContainerProvider;
import techreborn.client.container.builder.BuiltContainer;
import techreborn.client.container.builder.ContainerBuilder;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by modmuss50 on 20/06/2017.
 */
public class TileAutoCraftingTable extends TilePowerAcceptor implements IContainerProvider, IInventoryProvider, ISidedInventory {

	ResourceLocation currentRecipe;

	public Inventory inventory = new Inventory(10, "TileAutoCraftingTable", 64, this);
	public int progress;
	public int maxProgress = 120;
	public int euTick = 10;
	public Pair<ResourceLocation, IRecipe> cachedRecipe;
	public boolean customRecipe = true;
	InventoryCrafting inventoryCrafting = null;
	IRecipe lastCustomRecipe = null;

	public void setCurrentRecipe(IRecipe recipe, boolean customRecipe) {
		if (recipe != null) {
			currentRecipe = recipe.getRegistryName();
		} else {
			currentRecipe = null;
		}

		this.customRecipe = customRecipe;
		cachedRecipe = null;
	}

	public void setCurrentRecipe(ResourceLocation recipe, boolean customRecipe) {
		currentRecipe = recipe;
		this.customRecipe = customRecipe;
		cachedRecipe = null;
	}

	@Nullable
	public IRecipe getIRecipe() {
		if (customRecipe) {
			InventoryCrafting crafting = getCraftingInventory();
			for (IRecipe testRecipe : CraftingManager.field_193380_a) {
				if (testRecipe.func_77569_a(crafting, field_145850_b)) {
					return testRecipe;
				}
			}
		}
		if (currentRecipe == null) {
			return null;
		}
		if (cachedRecipe == null || !cachedRecipe.getLeft().equals(currentRecipe)) {
			IRecipe recipe = ForgeRegistries.RECIPES.getValue(currentRecipe);
			if (recipe != null) {
				cachedRecipe = Pair.of(currentRecipe, recipe);
				return recipe;
			}
			cachedRecipe = null;
			return null;
		}
		return cachedRecipe.getRight();
	}

	public InventoryCrafting getCraftingInventory() {
		if (inventoryCrafting == null) {
			inventoryCrafting = new InventoryCrafting(new Container() {
				@Override
				public boolean func_75145_c(EntityPlayer playerIn) {
					return false;
				}
			}, 3, 3);
		}
		for (int i = 0; i < 9; i++) {
			inventoryCrafting.func_70299_a(i, inventory.func_70301_a(i));
		}
		return inventoryCrafting;
	}

	@Override
	public void func_73660_a() {
		super.func_73660_a();
		if (field_145850_b.field_72995_K) {
			return;
		}
		IRecipe recipe = getIRecipe();
		if (recipe != null || customRecipe) {
			if (progress >= maxProgress) {
				if (make(recipe)) {
					progress = 0;
				}
			} else {
				if (canMake(recipe)) {
					if (canUseEnergy(euTick)) {
						progress++;
						useEnergy(euTick);
					}
				}
			}
		}
		if (recipe == null) {
			progress = 0;
		}
	}

	public boolean canMake(IRecipe recipe) {
		if (customRecipe) {
			recipe = getIRecipe();
		}
		if (recipe != null && recipe.func_194133_a(3, 3)) {
			boolean missingOutput = false;
			int[] stacksInSlots = new int[9];
			for (int i = 0; i < 9; i++) {
				stacksInSlots[i] = inventory.func_70301_a(i).func_190916_E();
			}
			for (Ingredient ingredient : recipe.func_192400_c()) {
				if (ingredient != Ingredient.field_193370_a) {
					boolean foundIngredient = false;
					for (int i = 0; i < 9; i++) {
						ItemStack stack = inventory.func_70301_a(i);
						int requiredSize = customRecipe ? 1 : 0;
						if (stacksInSlots[i] > requiredSize) {
							if (ingredient.apply(stack)) {
								foundIngredient = true;
								stacksInSlots[i]--;
								break;
							}
						}
					}
					if (!foundIngredient) {
						missingOutput = true;
					}
				}
			}
			if (!missingOutput) {
				if (hasOutputSpace(recipe.func_77571_b())) {
					return true;
				}
			}
			return false;
		}
		return false;
	}

	public boolean hasOutputSpace(ItemStack output) {
		ItemStack stack = inventory.func_70301_a(9);
		if (stack.func_190926_b()) {
			return true;
		}
		if (ItemUtils.isItemEqual(stack, output, true, true)) {
			if (stack.func_77976_d() > stack.func_190916_E() + output.func_190916_E()) {
				return true;
			}
		}
		return false;
	}

	public boolean make(IRecipe recipe) {
		if (canMake(recipe)) {
			if (recipe == null && customRecipe) {
				if (lastCustomRecipe == null) {
					return false;
				}//Should be uptodate as we just set it in canMake
				recipe = lastCustomRecipe;
			}
			for (int i = 0; i < recipe.func_192400_c().size(); i++) {
				Ingredient ingredient = recipe.func_192400_c().get(i);
				//Looks for the best slot to take it from
				ItemStack bestSlot = inventory.func_70301_a(i);
				if (ingredient.apply(bestSlot)) {
					bestSlot.func_190918_g(1);
				} else {
					for (int j = 0; j < 9; j++) {
						ItemStack stack = inventory.func_70301_a(j);
						if (ingredient.apply(stack)) {
							stack.func_190918_g(1); //TODO is this right? or do I need to use it as an actull crafting grid
							//TOOD check items to be retuned
							break;
						}
					}
				}
			}
			ItemStack output = inventory.func_70301_a(9);
			//TODO fire forge recipe event
			ItemStack ouputStack = recipe.func_77572_b(getCraftingInventory());
			if (output.func_190926_b()) {
				inventory.func_70299_a(9, ouputStack.func_77946_l());
			} else {
				//TODO use ouputStack in someway?
				output.func_190917_f(recipe.func_77571_b().func_190916_E());
			}
			return true;
		}
		return false;
	}

	public boolean hasIngredient(Ingredient ingredient) {
		for (int i = 0; i < 9; i++) {
			ItemStack stack = inventory.func_70301_a(i);
			if (ingredient.apply(stack)) {
				return true;
			}
		}
		return false;
	}

	public TileAutoCraftingTable() {
		super();
	}

	@Override
	public double getBaseMaxPower() {
		return 10000;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return 32;
	}

	@Override
	public boolean canAcceptEnergy(EnumFacing enumFacing) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(EnumFacing enumFacing) {
		return false;
	}

	@Override
	public BuiltContainer createContainer(EntityPlayer player) {
		return new ContainerBuilder("autocraftingTable").player(player.field_71071_by).inventory().hotbar()
			.addInventory().tile(this)
			.slot(0, 28, 25).slot(1, 46, 25).slot(2, 64, 25)
			.slot(3, 28, 43).slot(4, 46, 43).slot(5, 64, 43)
			.slot(6, 28, 61).slot(7, 46, 61).slot(8, 64, 61)
			.outputSlot(9, 145, 42).syncEnergyValue()
			.syncIntegerValue(this::getProgress, this::setProgress)
			.syncIntegerValue(this::getMaxProgress, this::setMaxProgress)
			.addInventory().create();
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	@Override
	public IInventory getInventory() {
		return inventory;
	}

	public int getProgress() {
		return progress;
	}

	public void setProgress(int progress) {
		this.progress = progress;
	}

	public int getMaxProgress() {
		if (maxProgress == 0) {
			maxProgress = 1;
		}
		return maxProgress;
	}

	public void setMaxProgress(int maxProgress) {
		this.maxProgress = maxProgress;
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tag) {
		if (currentRecipe != null) {
			tag.func_74778_a("currentRecipe", currentRecipe.toString());
		}
		tag.func_74757_a("customRecipe", customRecipe);
		return super.func_189515_b(tag);
	}

	@Override
	public void func_145839_a(NBTTagCompound tag) {
		if (tag.func_74764_b("currentRecipe")) {
			currentRecipe = new ResourceLocation(tag.func_74779_i("currentRecipe"));
		}
		customRecipe = tag.func_74767_n("customRecipe");
		super.func_145839_a(tag);
	}

	public boolean isItemValidForRecipeSlot(IRecipe recipe, ItemStack stack, int slotID) {
		if (recipe == null) {
			return true;
		}
		int bestSlot = findBestSlotForStack(recipe, stack);
		if (bestSlot != -1) {
			return bestSlot == slotID;
		}
		return true;
	}

	public int findBestSlotForStack(IRecipe recipe, ItemStack stack) {
		if (recipe == null) {
			return -1;
		}
		List<Integer> possibleSlots = new ArrayList<>();
		for (int i = 0; i < recipe.func_192400_c().size(); i++) {
			ItemStack stackInSlot = inventory.func_70301_a(i);
			Ingredient ingredient = recipe.func_192400_c().get(i);
			if (ingredient != Ingredient.field_193370_a && ingredient.apply(stack)) {
				if (stackInSlot.func_190926_b()) {
					possibleSlots.add(i);
				} else if (stackInSlot.func_77973_b() == stack.func_77973_b() && stackInSlot.func_77952_i() == stack.func_77952_i()) {
					if (stackInSlot.func_77976_d() >= stackInSlot.func_190916_E() + stack.func_190916_E()) {
						possibleSlots.add(i);
					}
				}
			}
		}
		//Slot, count
		Pair<Integer, Integer> smallestCount = null;
		for (Integer slot : possibleSlots) {
			ItemStack slotStack = inventory.func_70301_a(slot);
			if (slotStack.func_190926_b()) {
				return slot;
			}
			if (smallestCount == null) {
				smallestCount = Pair.of(slot, slotStack.func_190916_E());
			} else if (smallestCount.getRight() >= slotStack.func_190916_E()) {
				smallestCount = Pair.of(slot, slotStack.func_190916_E());
			}
		}
		if (smallestCount != null) {
			return smallestCount.getLeft();
		}
		return -1;
	}

	@Override
	public boolean func_94041_b(int index, ItemStack stack) {
		int bestSlot = findBestSlotForStack(getIRecipe(), stack);
		if (bestSlot != -1) {
			return index == bestSlot;
		}
		return super.func_94041_b(index, stack);
	}

	@Override
	public boolean func_180462_a(int index, ItemStack itemStackIn, EnumFacing direction) {
		if (index == 9) {
			return false;
		}
		int bestSlot = findBestSlotForStack(getIRecipe(), itemStackIn);
		if (bestSlot != -1) {
			return index == bestSlot;
		}
		return false;
	}

	@Override
	public int[] func_180463_a(EnumFacing side) {
		return new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
	}

	@Override
	public boolean func_180461_b(int index, ItemStack stack, EnumFacing direction) {
		return index == 9;
	}
}
